/*
 * NiuBi Pocket Router board support
 *
 * Copyright (C) 2018 L D Pinney <ldpinney@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 */

#include <asm/mach-ath79/ath79.h>
#include <asm/mach-ath79/ar71xx_regs.h>

#include <linux/platform_device.h>
#include <linux/gpio.h>

#include "common.h"
#include "dev-eth.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-spi.h"
#include "dev-usb.h"
#include "dev-wmac.h"
#include "machtypes.h"

#define NB_PR1_GPIO_LED_WLAN	0
#define NB_PR1_GPIO_LED_STATUS	4
#define NB_PR1_GPIO_SPI_CS1		11
#define NB_PR1_GPIO_BTN_RESET	12
#define NB_PR1_GPIO_LED_LAN		13
#define NB_PR1_GPIO_BTN_WPS		16
#define NB_PR1_MAC0_OFFSET		0x0
#define NB_PR1_MAC1_OFFSET		0x6
#define NB_PR1_WMAC_CALDATA_OFFSET	0x1000
#define NB_PR1_KEYS_POLL_INTERVAL		20	/* msecs */
#define NB_PR1_KEYS_DEBOUNCE_INTERVAL	(3 * NB_PR1_KEYS_POLL_INTERVAL)


static struct gpio_led nb_pr1_leds_gpio[] __initdata = {
	{
		.name = "nb-pr1:red:status",
		.gpio = NB_PR1_GPIO_LED_STATUS,
		.active_low = 1,
	},
	{
		.name = "nb-pr1:green:network",
		.gpio = NB_PR1_GPIO_LED_LAN,
		.active_low = 1,
	},
	{
		.name = "nb-pr1:blue:wifi",
		.gpio = NB_PR1_GPIO_LED_WLAN,
		.active_low = 1,
	},
};

static struct gpio_keys_button nb_pr1_gpio_keys[] __initdata = {
	{
		.desc = "RESET",
		.type = EV_KEY,
		.code = KEY_RESTART,
		.debounce_interval = NB_PR1_KEYS_DEBOUNCE_INTERVAL,
		.gpio = NB_PR1_GPIO_BTN_RESET,
		.active_low = 1,
	},
	{
		.desc = "WPS",
		.type = EV_KEY,
		.code = BTN_0,
		.debounce_interval = NB_PR1_KEYS_DEBOUNCE_INTERVAL,
		.gpio = NB_PR1_GPIO_BTN_WPS,
		.active_low = 0,
	},
};

static void __init nb_pr1_common_setup(void)
{
	u8 *art = (u8 *) KSEG1ADDR(0x1fff0000);

	ath79_register_m25p80(NULL);

	ath79_setup_ar933x_phy4_switch(false, false);

	ath79_register_usb();
	ath79_register_mdio(0, 0x0);

	ath79_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_MII;
	ath79_eth0_data.speed = SPEED_100;
	ath79_eth0_data.duplex = DUPLEX_FULL;
	ath79_eth0_data.phy_mask = BIT(4);
	ath79_init_mac(ath79_eth0_data.mac_addr, art + NB_PR1_MAC0_OFFSET, 0);
	ath79_register_eth(0);


	ath79_eth1_data.phy_if_mode = PHY_INTERFACE_MODE_GMII;
	ath79_eth1_data.speed = SPEED_1000;
	ath79_eth1_data.duplex = DUPLEX_FULL;
	ath79_switch_data.phy_poll_mask |= BIT(4);
	ath79_switch_data.phy4_mii_en = 1;
	ath79_init_mac(ath79_eth1_data.mac_addr, art + NB_PR1_MAC1_OFFSET, 0);
	ath79_register_wmac(art + NB_PR1_WMAC_CALDATA_OFFSET, NULL);
	ath79_register_eth(1);

	ath79_register_leds_gpio(-1, ARRAY_SIZE(nb_pr1_leds_gpio),
				 nb_pr1_leds_gpio);

	ath79_register_gpio_keys_polled(-1, NB_PR1_KEYS_POLL_INTERVAL,
					ARRAY_SIZE(nb_pr1_gpio_keys),
					nb_pr1_gpio_keys);
}

static void __init nb_pr1_setup(void)
{
	nb_pr1_common_setup();
}

MIPS_MACHINE(ATH79_MACH_NB_PR1, "NB-PR1", "NiuBi Pocket Router One",
	     nb_pr1_setup);
